from typing import List

from blue_options.terminal import show_usage, xtra


def help_get_static_image(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun", mono=mono)

    args = [
        "[--lat <lat>]",
        "[--lon <lon>]",
        "[--filename <filename>]",
        "[--zoom <20>]",
        "[--maptype satellite]",
        "[--size 640x640]",
    ]

    return show_usage(
        [
            "@gmaps",
            "get_static_image",
            f"[{options}]",
            "[-|<object-name>]",
        ]
        + args,
        "get Google Maps Static Image.",
        {
            "https://developers.google.com/maps/documentation/maps-static/start": [],
        },
        mono=mono,
    )



from typing import Tuple, Any
from roboflow import Roboflow
import pprint

from blueness import module
from blue_options.elapsed_timer import ElapsedTimer

from roofai import NAME
from roofai.env import ROBOFLOW_API_KEY
from roofai.logger import logger


NAME = module.name(__file__, NAME)


def get_status() -> Tuple[bool, Any]:
    rf = Roboflow(api_key=ROBOFLOW_API_KEY)

    try:
        status = rf.workspace()
    except Exception as e:
        logger.error(e)
        return False, {}

    print(status)

    return True, status



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit
from blue_options.logger import crash_report
from blue_objects import path

from roofai import NAME
from roofai.semseg.interface import predict, train
from roofai.semseg import Profile
from roofai.logger import logger

NAME = module.name(__file__, NAME)


list_of_tasks = "predict|train"

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=list_of_tasks,
)
parser.add_argument(
    "--activation",
    type=str,
    default="sigmoid",
    help="sigmoid or None for logits or softmax2d for multi-class segmentation",
)
parser.add_argument(
    "--classes",
    type=str,
    default="",
    help="one+two+three+four",
)
parser.add_argument(
    "--dataset_path",
    type=str,
)
parser.add_argument(
    "--device",
    type=str,
    default="cpu",
    help="cpu|cuda",
)
parser.add_argument(
    "--encoder_name",
    type=str,
    default="se_resnext50_32x4d",
)
parser.add_argument(
    "--encoder_weights",
    type=str,
    default="imagenet",
)
parser.add_argument(
    "--model_path",
    type=str,
)
parser.add_argument(
    "--prediction_path",
    type=str,
)
parser.add_argument(
    "--profile",
    type=str,
    default="VALIDATION",
    help="FULL|QUICK|VALIDATION",
)
parser.add_argument(
    "--epoch_count",
    type=int,
    default=-1,
    help="-1: according to profile",
)
args = parser.parse_args()

success = True
try:
    profile = Profile[args.profile]
    logger.info(f"profile: {profile}")
except:
    crash_report(f"bad profile: {args.profile}")
    success = False

if success:
    success = args.task in list_of_tasks.split("|")
    if args.task == "predict":
        if not path.create(args.prediction_path):
            success = False
        else:
            predict(
                model_path=args.model_path,
                dataset_path=args.dataset_path,
                prediction_path=args.prediction_path,
                device=args.device,
                profile=profile,
            )
    elif args.task == "train":
        train(
            dataset_path=args.dataset_path,
            model_path=args.model_path,
            encoder_name=args.encoder_name,
            encoder_weights=args.encoder_weights,
            classes=args.classes.split("+"),
            activation=args.activation,
            device=args.device,
            profile=profile,
            epoch_count=args.epoch_count,
        )
    else:
        success = None

sys_exit(logger, NAME, args.task, success)



from roboflow import Roboflow
import glob
from tqdm import tqdm

from blueness import module
from blue_options.elapsed_timer import ElapsedTimer
from blue_objects import objects

from roofai import NAME
from roofai.env import ROBOFLOW_API_KEY
from roofai.roboflow.create import create_project
from roofai.logger import logger


NAME = module.name(__file__, NAME)


# https://docs.roboflow.com/api-reference/images/upload-api
def upload_to_project(
    object_name: str,
    project_name: str,
    create: bool = True,
    verbose: bool = False,
) -> bool:
    logger.info(
        "{}.upload_to_project: {} -> {}".format(
            NAME,
            object_name,
            project_name,
        )
    )

    if create:
        if not create_project(
            project_name=project_name,
            project_description="uploaded-from-{}".format(
                object_name.replace("_", "-")
            ),
            project_type="semantic-segmentation",
            project_license="MIT",
        ):
            return False

    timer = ElapsedTimer()

    rf = Roboflow(api_key=ROBOFLOW_API_KEY)
    project = rf.workspace().project(project_name)

    for filename in tqdm(
        glob.glob(
            objects.path_of(
                object_name=object_name,
                filename="*.png",
            )
        )
    ):
        logger.info(filename)

        try:
            project.upload(
                image_path=filename,
                batch_name=object_name,
                split="train",
                num_retry_uploads=3,
                # sequence_number=99,
                # sequence_size=100,
            )

        # - sequence_number: [Optional] If you want to keep the order of your images in the dataset, pass sequence_number and sequence_size..
        # - sequence_size: [Optional] The total number of images in the sequence. Defaults to 100,000 if not set.

        except Exception as e:
            logger.error(e)
            return False

    timer.stop()
    logger.info(f"took {timer.elapsed_pretty()}")

    return True



"""
Copied with modification from ../../notebooks/semseg.ipynb
"""

import os
import time
import numpy as np
import torch
import cv2
from tqdm import tqdm
from scipy import ndimage
import segmentation_models_pytorch as smp
from shapely.geometry import Polygon

from blue_options import string
from blue_objects import file, path
from blue_objects.graphics.gif import generate_animated_gif

from roofai.semseg.augmentation import get_validation_augmentation, get_preprocessing
from roofai.dataset.classes import RoofAIDataset
from roofai.semseg.dataloader import Dataset
from roofai.semseg.utils import visualize
from roofai.semseg import Profile
from roofai.logger import logger


class SemSegModel:
    def __init__(
        self,
        model_filename: str,
        profile: Profile = Profile.VALIDATION,
        device="cpu",  # 'cuda'
        tolerance: float = 2,
    ):
        self.profile = profile
        self.filename = model_filename
        self.tolerance = tolerance
        self.device = device

        logger.info(
            "{}.load({}) on {}: {}".format(
                self.__class__.__name__,
                self.filename,
                self.device,
                self.profile,
            )
        )

        self.model = torch.load(
            self.filename,
            map_location=torch.device(self.device),
            weights_only=False,
        ).to(self.device)

        success, metadata = file.load_json(
            file.add_extension(
                self.filename,
                "json",
            )
        )
        assert success

        self.encoder_name = metadata["encoder_name"]
        self.encoder_weights = metadata["encoder_weights"]
        self.classes = metadata["classes"]
        self.activation = metadata["activation"]

        logger.info(self.signature)

    @property
    def object_name(self):
        return path.name(file.path(self.filename))

    def predict(
        self,
        dataset_path,
        output_path,
        in_notebook: bool = False,
    ):
        dataset = RoofAIDataset(dataset_path)

        logger.info(
            "{}.predict({}:{}) -{}-> {}".format(
                self.__class__.__name__,
                dataset.source,
                dataset.dataset_path,
                self.device,
                output_path,
            )
        )

        x_test_dir = os.path.join(dataset.dataset_path, "test")
        y_test_dir = os.path.join(dataset.dataset_path, "testannot")

        # test dataset without transformations for image visualization
        test_dataset_vis = Dataset(
            x_test_dir,
            y_test_dir,
            classes=self.classes,
            count=self.profile.data_count,
        )

        preprocessing_fn = smp.encoders.get_preprocessing_fn(
            self.encoder_name,
            self.encoder_weights,
        )

        test_dataset = Dataset(
            x_test_dir,
            y_test_dir,
            augmentation=get_validation_augmentation(),
            preprocessing=get_preprocessing(preprocessing_fn),
            classes=self.classes,
            count=self.profile.data_count,
        )

        list_of_images = []
        index_list = (
            [np.random.choice(len(test_dataset))]
            if self.profile == Profile.VALIDATION
            else range(len(test_dataset))
        )
        for n in tqdm(index_list):
            image_vis = test_dataset_vis[n][0].astype("uint8")
            image, gt_mask = test_dataset[n]

            gt_mask = gt_mask.squeeze()

            start_time = time.time()
            x_tensor = torch.from_numpy(image).to(self.device).unsqueeze(0)
            pr_mask = self.model.predict(x_tensor)
            pr_mask = pr_mask.squeeze().cpu().numpy().round()
            elapsed_time = time.time() - start_time
            logger.info(
                "took {}".format(
                    string.pretty_duration(
                        elapsed_time,
                        include_ms=True,
                    )
                )
            )

            label_mask, label_count = ndimage.label(pr_mask)
            logger.info(f"{label_count} object(s) found.")
            list_of_contours = []
            for label in range(1, label_count + 1):
                mask = label_mask == label

                contour, _ = cv2.findContours(
                    mask.astype(np.uint8),
                    cv2.RETR_EXTERNAL,
                    cv2.CHAIN_APPROX_SIMPLE,
                )

                if contour[0].shape[0] <= 4:
                    continue

                polygon = Polygon(
                    [
                        (x, y)
                        for x, y in zip(
                            contour[0][:, 0, 0],
                            contour[0][:, 0, 1],
                        )
                    ]
                )
                polygon = polygon.simplify(self.tolerance)

                list_of_contours.append(
                    (
                        list(polygon.exterior.xy[0]),
                        list(polygon.exterior.xy[1]),
                    )
                )

            file.save_json(
                os.path.join(output_path, f"predict-{n:05d}.json"),
                {
                    "count": label_count,
                    "contour": list_of_contours,
                    "device": self.device,
                    "elapsed_time": elapsed_time,
                },
            )

            filename = os.path.join(output_path, f"predict-{n:05d}.png")
            visualize(
                {
                    "image": image_vis,
                    "groundtruth": gt_mask,
                    "prediction": pr_mask,
                },
                in_notebook=in_notebook,
                filename=filename,
                description=[
                    f"model: {self.object_name}",
                    self.signature,
                    f"device: {self.device}",
                    f"{label_count} object(s)",
                    "took {}".format(
                        string.pretty_duration(
                            elapsed_time,
                            include_ms=True,
                            short=True,
                            largest=True,
                        )
                    ),
                ],
                list_of_contours=list_of_contours,
            )
            list_of_images += [filename]

        generate_animated_gif(
            list_of_images,
            os.path.join(output_path, "predict.gif"),
            frame_duration=500,
        )

    @property
    def signature(self):
        return "{}: {}[{}]-{}-> {} @ {:.2f} px".format(
            self.__class__.__name__,
            self.encoder_name,
            self.encoder_weights,
            self.activation,
            ",".join(self.classes),
            self.tolerance,
        )
